# Generated by Django 1.10.5 on 2017-01-23 11:41

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix", "0009_delete_corporateimage"),
    ]
