# Generated by Django 1.10.6 on 2017-04-28 08:50

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix", "0016_auto_20170419_1533"),
    ]

    operations = [
        migrations.AlterField(
            model_name="log",
            name="action_flag",
            field=models.PositiveSmallIntegerField(
                choices=[(1, "Add"), (2, "Change"), (3, "Delete")],
                verbose_name="Action",
            ),
        ),
    ]
