# Generated by Django 1.10.6 on 2017-04-19 15:33

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix", "0015_auto_20170418_1515"),
    ]

    operations = [
        migrations.AlterField(
            model_name="log",
            name="action_flag",
            field=models.PositiveSmallIntegerField(
                choices=[(1, "A\xf1adir"), (2, "Cambiar"), (3, "Borrar")],
                verbose_name="Acci\xf3n",
            ),
        ),
    ]
