# Generated by Django 1.10.5 on 2017-01-23 10:14
# Simplified by Juanmi Taboada on 2017-14-02 11:12

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix", "0008_auto_20170119_1526"),
    ]

    operations = []
