# Generated by Django 1.10.6 on 2017-07-13 15:58

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("codenerix", "0017_auto_20170428_0850"),
    ]

    operations = [
        migrations.AddField(
            model_name="log",
            name="snapshot_txt",
            field=models.TextField(
                blank=True,
                verbose_name=b"Snapshot Txt",  # type: ignore[arg-type]
            ),
        ),
    ]
